<?php

namespace RotinaEstoque\Controllers\Planning;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
// use \Teknisa\Model\Planejamento\Entities\Cardadia;
// use \Teknisa\Model\Estoque\Entities\Lanctoestoq;
// use \Teknisa\Model\Planejamento\Entities\Retirada;
// use \Teknisa\Model\Planejamento\Entities\Itemreti;
// use \Teknisa\Model\Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Lanctoestoq;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;

class HomeController {
  protected $crudService;
  protected $validacoesEstoqueService;
  const COD_ERROR_PEBBIAN = 1500;
  
  public function __construct($connection, $environment, $util, $service, $crudService, ValidacoesEstoqueService $validacoesEstoqueService) {
    $this->connection   = $connection;
    $this->environment  = $environment;
    $this->util         = $util;
    $this->service      = $service;
    $this->crudService	= $crudService;
    $this->validacoesEstoqueService = $validacoesEstoqueService;
  }
  
  public function testConnection(Request\Row $request, Response $response) {
      $data = $this->service->testConnection();
      $response->addDataSet(new DataSet('connection', $data));
    }

  public function getInitialData(Request\Row $request, Response $response) {
    $data = array(
      'NRORG'         => $this->environment->getNrOrg(),
      'LOGGED_BRANCH' => $this->environment->getCdFilial()
    );
    $response->addDataSet(new DataSet('initialData', $data));
  }
  
  public function getCdNvProduto(Request\Row $request, Response $response) {
    $data = $this->service->getCdNvProduto();
    $response->addDataSet(new DataSet('cdNvProduto', $data));
  }
  
  public function loadUnitParameters(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $params = [
      "CDFILIAL" => $row["CDFILIAL"]["CDFILIAL"],
      "NRORG"    => $this->environment->getNrOrg()
    ];
    $data = $this->service->planningWithdrawalData($params);
    $response->addDataSet(new DataSet("params", $data));
  }

  public function planningWithdrawalData(Request\Row $request, Response $response) {
    $selectedBranch = $request->getParameters();
    $CDFILIAL = $selectedBranch["CDFILIAL"]["CDFILIAL"];
    try {
      $params = [
        "CDFILIAL" => $CDFILIAL,
        "NRORG"    => $this->environment->getNrOrg()
      ];
      
      $data = $this->service->planningWithdrawalData($params);
    } catch(Exception $e) {
        throw $e;
    }
    
    $response->addDataSet(new DataSet('planningWithdrawalData', $data)); 
  }
  
  public function prevreti(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $params = [
      "CDFILIAL" => $row["CDFILIAL"],
      "CDSERVFILI" => $row["CDSERVFILI"],
      "DTPREVRETI" => $row["DTPREVRETI"],
      "IDPREVRETI"    => $row["IDPREVRETI"]
    ];
    $data = $this->service->prevreti($params);
    $response->addDataSet(new DataSet("params", $data));
  }

  public function servico(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $params = [
      "CDFILIAL" => $row["CDFILIAL"],
      "CDSERVFILI" => $row["CDSERVFILI"]
    ];
    $data = $this->service->servico($params);
    $response->addDataSet(new DataSet("params", $data));
  }

  public function loadMessage(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $params = [
      "CDFILIAL" => $row["CDFILIAL"],
      "CDSERVFILI" => $row["CDSERVFILI"],
      "DTMENSREQU" => $row["DTPREVRETI"]
    ];
    $data = $this->service->loadMessage($params);
    $response->addDataSet(new DataSet("params", $data));
  }

  public function verifyProduct(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $params = [
      "CDFILIAL" => $row["CDFILIAL"],
      "CDPRODUTO" => $row["CDPRODUTO"]
    ];
    $data = $this->service->verifyProduct($params);
    $response->addDataSet(new DataSet("params", $data));
  }

  public function getUserInfo(Request\Row $request, Response $response) {
    $data = $this->service->userInfo();
    $response->addDataSet($data);
  }

  public function calculaRequisicaoServico(Request\Row $request, Response $response) {
		try {
			$row = $request->getParameters();
			$params = [
			  "NRORG" => $row['NRORG'],
			  "CDOPERADOR" => $row['CDOPERADOR'],
			  "CDFILIAL" => $row['conditions']['CDFILIAL'],
			  "SERVICOS" => $row['conditions']['CDSERVFILI'],
			  "DTINICIAL" => $row['conditions']['DTPREVRETI'],
			  "DTFINAL" => $row['conditions']['DTPREVRETI'],
			  "DTHRCALCPR" => $row['DTHRCALCPR'],
			  "PEACRESCIMO" => $row['PEACRESCIMO'],
			  "STDTINICIAL" => $row['STDTINICIAL'],
			  "STDTFINAL" => $row['STDTFINAL'],
			  "IDARREDOND" => $row['IDARREDOND'],
			  "IDPRODUTOS" => $row['IDPRODUTOS'],
			];
      $data = $this->service->calculaRequisicao($params);
      $response->addDataSet(new DataSet("nrconsplan", $data));
		} catch (Exception $e) {
       $response->setError(new Error($e->getMessage(), 1));
    }

	}

  public function addProduct(Request\Row $request, Response $response) {
		try {
			$row = $request->getParameters();
			$params = [
			  "NRORG" => $row['NRORG'],
			  "CDOPERADOR" => $row['CDOPERADOR'],
			  "CDFILIAL" => $row['conditions']['CDFILIAL'],
			  "SERVICOS" => $row['conditions']['CDSERVFILI'],
			  "DTINICIAL" => $row['conditions']['DTPREVRETI'],
			  "DTFINAL" => $row['conditions']['DTPREVRETI'],
			  "DTHRCALCPR" => $row['DTHRCALCPR'],
			  "PEACRESCIMO" => $row['PEACRESCIMO'],
			  "STDTINICIAL" => $row['STDTINICIAL'],
			  "STDTFINAL" => $row['STDTFINAL'],
			  "IDARREDOND" => $row['IDARREDOND'],
			  "IDPRODUTOS" => $row['IDPRODUTOS'],
			];
      $data = $this->service->addProduct($params);
      $response->addDataSet(new DataSet("nrconsplan", $data));
		} catch (Exception $e) {
       $response->setError(new Error($e->getMessage(), 1));
    }
	}
}