<?php

namespace RotinaEstoque\Controllers\Planning;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Teknisa\Model\Estoque\Entities\Prevreti;


class ServiceRequestController {
    public function __construct($entityManager, $environment, $connection, $libUtilities){
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->libUtilities  = $libUtilities;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
	public function excluiPrato(Request\Row $request, Response $response) {
		$this->beginTransaction();
		try {
			$rows = $request->getParameters();
        	foreach ($rows as $prato) {
    			$prato['DTPREVRETI'] = $this->converteParaDateTime($prato['DTPREVRETI']);
                if($prato['CDTIPOPRAT'] =='null' || $prato['CDTIPOPRAT'] =='undefined'){
                    $prato = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prevreti')
	                ->findOneBy(array("cdfilial" => $prato['CDFILIAL'], "cdservfili" => $prato['CDSERVFILI'], "dtprevreti" => $prato['DTPREVRETI'], "idprevreti" => $prato['IDPREVRETI'], "nrorg" => $prato['NRORG'], "cdproduto" => $prato['CDPRODUTO']));
	                $this->entityManager->remove($prato);
                }else{
                    if($prato['CDPRODUTO'] =='null' || $prato['CDPRODUTO'] =='undefined'){
                        // adicionar prato aux
                        $prato = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prevreti')
    	                ->findBy(array("cdfilial" => $prato['CDFILIAL'], "cdservfili" => $prato['CDSERVFILI'], "dtprevreti" => $prato['DTPREVRETI'], "idprevreti" => $prato['IDPREVRETI'], "nrorg" => $prato['NRORG'], "cdprato" => $prato['CDPRATO'], "cdtipoprat" => $prato['CDTIPOPRAT']));
    		            foreach ($prato as $produto) {
    		            	$this->entityManager->remove($produto);
    		            }                        
                    }else{
                        // adicionar prato aux
                        $prato = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prevreti')
    	                ->findOneBy(array("cdfilial" => $prato['CDFILIAL'], "cdservfili" => $prato['CDSERVFILI'], "dtprevreti" => $prato['DTPREVRETI'], "idprevreti" => $prato['IDPREVRETI'], "nrorg" => $prato['NRORG'], "cdprato" => $prato['CDPRATO'], "cdtipoprat" => $prato['CDTIPOPRAT'], "cdproduto" => $prato['CDPRODUTO']));
                        $this->entityManager->remove($prato);
                    }
                }
        	}
        	$this->entityManager->flush();
			$this->commit();
		} catch (\Exception $e) {
			$this->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }

	}
	
	public function createProduct(DTO\Request\Row $request, DTO\Response $response) {
		try {
            // pegar todos os dados da row pelo prevreti produto
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();
			$row['DTPREVRETI'] = $this->converteParaDateTime($row['DTPREVRETI']);
			$row['DTLIMITERETI'] = $this->converteParaDateTime($row['DTLIMITERETI']);
			$validate = [];
			if(empty($validate)){
				if(!empty($row['CDTIPOPRAT'])){
					$prevreti = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prevreti')
		            ->findOneBy(array("cdfilial"     => $row['CDFILIAL'], 
		            				  "cdservfili"   => $row['CDSERVFILI'], 
		            				  "dtprevreti"   => $row['DTPREVRETI'], 
		            				  "nrrequextr"   => $row['NRREQUEXTR'], 
		            				  "idprevreti"   => $row['IDPREVRETI'], 
		            				  "cdproduto"    => $row['CDPRODUTO'],
		            				  "cdtipoprat"   => $row['CDTIPOPRAT'],
		            				  "cdfilauxprat" => $row['CDFILAUXPRAT'],
		            				  "cdprato" => $row['CDPRATO']
		            				  ));
		    	}
		    	else{
		    		$prevreti = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prevreti')
		            ->findOneBy(array("cdfilial"     => $row['CDFILIAL'], 
		            				  "cdservfili"   => $row['CDSERVFILI'], 
		            				  "dtprevreti"   => $row['DTPREVRETI'], 
		            				  "nrrequextr"   => $row['NRREQUEXTR'], 
		            				  "idprevreti"   => $row['IDPREVRETI'], 
		            				  "cdproduto"    => $row['CDPRODUTO']
		            				  ));
		    	}
	            if (empty($prevreti)) {
	               // insert
					$newPrevreti = new \Teknisa\Model\Planejamento\Entities\Prevreti();
					$newPrevreti = $this->constroiPrevreti($newPrevreti, $row);
		            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
					$this->entityManager->persist($newPrevreti);
					$this->entityManager->flush();
				

	            } else {
	               // update
		            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
					$this->entityManager->persist($this->updatePrevreti($prevreti, $row));
					$this->entityManager->flush();

	            }
				}
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }

	}

	public function createMessage(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();
			$row['DTMENSREQU'] = $this->converteParaDateTime($row['DTMENSREQU']);
    		$message = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Mensrequ')
	            ->findOneBy(array(
	                "cdfilial"     => $row['CDFILIAL'],
	                "cdservfili"   => $row['CDSERVFILI'],
				    "dtmensrequ"   => $row['DTMENSREQU'],
			        ));
			if(empty($message)){
    			$newMessage = new \Teknisa\Model\Planejamento\Entities\Mensrequ();
				$newMessage = $this->constroiMensagem($newMessage, $row);
	            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
				$this->entityManager->persist($newMessage);
				$this->entityManager->flush();
			}else{
	            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
				$this->entityManager->persist($this->updateMessage($message, $row));
				$this->entityManager->flush();
			}
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }

	}

    public function validGrupOper(DTO\Request\Row $request, DTO\Response $response){
        try {
            $cdGrupOper = $this->getGrupOper();
            $cdOperador = $this->environment->getCdOperador();
        	//Verifica Grupo de operdores cadastrado na tela de parâmetros.
        	$sqlGrupo = "SELECT NRPERFILACESSO FROM AUTPERFILACESSO WHERE IDAUTPERFACESSO = 'ALTREQUSERV'";
        	$queryGrupo  = $this->entityManager->getConnection()->prepare($sqlGrupo);
            $queryGrupo->execute();
            $resultGrupo = $queryGrupo->fetchAll();

            if(!empty($resultGrupo)){
            	$resultGrupo = $resultGrupo[0]["NRPERFILACESSO"];
            	
    	    //Verifica se Operador esta no Grupo cadastrado (Mobile)
    	    $sqlOperGrup = "SELECT
    							OPERADOR.CDOPERADOR,
    							OPERADOR.NMOPERADOR,
    							PERFACESSPARC.NRPERFILACESSO
    						FROM PERFACESSPARC
    						JOIN OPERADOR 
    							ON ( PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
    							AND PERFACESSPARC.NRORG = OPERADOR.NRORG )
    						WHERE NRPERFILACESSO = :NRPERFILACESSO
    							AND  OPERADOR.CDOPERADOR = :OPERADOR";
    	    $paramsOperGrup = array(
    	    	"NRPERFILACESSO" => $resultGrupo,	
    	    	"OPERADOR" => $cdOperador
    	    );
    	    $queryOperGrup  = $this->entityManager->getConnection()->prepare($sqlOperGrup);
    	    $queryOperGrup->execute($paramsOperGrup);
    	    $resultOperGrup = $queryOperGrup->fetchAll();
    	    if(!empty($resultOperGrup)){
    	        $resultOperGrup = $resultOperGrup[0]["CDOPERADOR"];
    	       }
            }
        	if(empty($resultGrupo)){ //se nao tem perfil de acesso no mobile, verifica no delphi
    	    	if(empty($cdGrupOper)){ //se tambem nao tem grupo de operadores cadastrado no delphi, entao todo mundo esta autorizado
    	    		$response->addDataSet(new DataSet ('authorized', array($cdOperador)));
    	    	}else{
    		    	//Verifica se Operador esta no Grupo cadastrado (Delphi)
    		    	$sql = "SELECT * FROM OPERGRUPOP WHERE CDGRUPOPER = :CDGRUPOPER AND CDOPERADOR = :CDOPERADOR";
    		    	$params = array(
    		    		"CDGRUPOPER" => $cdGrupOper,	
    		    		"CDOPERADOR" => $cdOperador
    		    	);
    		    	$query  = $this->entityManager->getConnection()->prepare($sql);
    		        $query->execute($params);
    		        $result = $query->fetchAll();
    	    		
    	    		if(!empty($result)){ //se operador esta no grupo do delphi então aprova, senão nao autoriza
    	    			$response->addDataSet(new DataSet ('authorized', array($cdOperador)));
    	    		}else{
    	    			$response->addDataSet(new DataSet ('denied', array($cdOperador)));
    	    		}
    	    	}
        	}else{
        		if(!empty($resultOperGrup)){ //se tem perfil de acesso cadastrado no mobile e o operador faz parte, aprova, senão reprova
        			$response->addDataSet(new DataSet ('authorized', array($cdOperador)));
        		}else{
        			$response->addDataSet(new DataSet ('denied', array($cdOperador)));
        		}
        	}
        } catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
        	$response->addDataSet(new DataSet ('denied', array($cdOperador)));
        }   
    }

    private function getGrupOper(){
    	$sql = "SELECT CDGROPERRESE FROM PARAMGERAL";
    	$query  = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        return $query->fetchAll()[0]["CDGROPERRESE"];
    }

	public function constroiPrevreti($prevreti, $row) {
	 	$row['DTINCLUSAO'] = empty($row['DTINCLUSAO']) ? new DateTime() : $this->converteParaDateTime(explode(' ', $row['DTINCLUSAO'])[0]);
	 	$row['DTULTATU'] = $this->converteParaDateTime(explode(' ', $row['DTULTATU'])[0]);

        $prevreti->setCdoperador($row['CDOPERADOR']);
        $prevreti->setCdoperinclusao($row['CDOPERINCLUSAO']);
        $prevreti->setCdoperultatu($row['CDOPERULTATU']);
        $prevreti->setNrorginclusao($row['NRORGINCLUSAO']);
        $prevreti->setNrorg($row['NRORG']);
        
        $prevreti->setCdfilial($row['CDFILIAL']);
        $prevreti->setCdservfili($row['CDSERVFILI']);
        $prevreti->setDtprevreti($row['DTPREVRETI']);
        $prevreti->setIdprevreti($row['IDPREVRETI']);
        $prevreti->setCdprato($row['CDPRATO']);
        $prevreti->setCdtipoprat($row['CDTIPOPRAT']);
        $prevreti->setNrrequextr($row['NRREQUEXTR']);
        $prevreti->setNrprprevreti($row['NRPRPREVRETI']);
        $prevreti->setCdproduto($row['CDPRODUTO']);
        $prevreti->setDtlimitereti($row['DTLIMITERETI']);
        $prevreti->setHrlimitereti($row['HRLIMITERETI']);
        $prevreti->setQtprevreti($row['QTPREVRETI']);
        $prevreti->setQtprevextra($row['QTPREVEXTRA']);
        $prevreti->setQtprevdevol($row['QTPREVDEVOL']);
        $prevreti->setQtprevlib($row['QTPREVLIB']);
        $prevreti->setCdfilauxprat($row['CDFILAUXPRAT']);
        $prevreti->setIdativo($row['IDATIVO']);
        $prevreti->setIdprevtpprat($row['IDUTRETIPRAT']);
        $prevreti->setNrorgultatu($row['NRORGULTATU']);
        $prevreti->setDtinclusao($row['DTINCLUSAO']);
        $prevreti->setDtultatu($row['DTULTATU']);

        return $prevreti;
    }

	public function updatePrevreti($prevreti, $row) {
	 	$row['DTINCLUSAO'] = empty($row['DTINCLUSAO']) ? new DateTime() : $this->converteParaDateTime(explode(' ', $row['DTINCLUSAO'])[0]);
	 	$row['DTULTATU'] = empty($row['DTULTATU']) ? new DateTime() : $this->converteParaDateTime(explode(' ', $row['DTULTATU'])[0]);
        $prevreti->setDtlimitereti($row['DTLIMITERETI']);
        $prevreti->setHrlimitereti($row['HRLIMITERETI']);
        $prevreti->setQtprevreti($row['QTPREVRETI']);
        $prevreti->setQtprevextra($row['QTPREVEXTRA']);
        $prevreti->setQtprevdevol($row['QTPREVDEVOL']);
        $prevreti->setQtprevlib($row['QTPREVLIB']);
        $prevreti->setNrorgultatu($row['NRORGULTATU']);
        $prevreti->setDtinclusao($row['DTINCLUSAO']);
        $prevreti->setDtultatu($row['DTULTATU']);

        return $prevreti;
    }

	public function constroiMensagem($newMessage, $row) {
	 	$row['DTINCLUSAO'] = new DateTime();

        $newMessage->setCdfilial($row['CDFILIAL']);
        $newMessage->setCdservfili($row['CDSERVFILI']);
        $newMessage->setDtmensrequ($row['DTMENSREQU']);
        $newMessage->setNrsequrequ($row['NRSEQUREQU']);
        $newMessage->setDsobsrequ($row['DSOBSREQU']);
        $newMessage->setNrorg($row['NRORG']);
        $newMessage->setDtinclusao($row['DTINCLUSAO']);
        $newMessage->setDtultatu($row['DTINCLUSAO']);
        $newMessage->setNrorginclusao($row['NRORG']);
        $newMessage->setCdoperinclusao($row['CDOPERADOR']);
        $newMessage->setNrorgultatu($row['NRORG']);
        $newMessage->setCdoperultatu($row['CDOPERADOR']);
        $newMessage->setIdativo($row['IDATIVO']);

        return $newMessage;
    }

	public function updateMessage($newMessage, $row) {
	 	$row['DTULTATU'] = new DateTime();
        $newMessage->setDsobsrequ($row['DSOBSREQU']);
        $newMessage->setNrorgultatu($row['NRORG']);
        $newMessage->setDtultatu($row['DTULTATU']);
        $newMessage->setCdoperultatu($row['CDOPERADOR']);

        return $newMessage;
    }

    private function validaProduto($row){
    	$validate = [];
    	$condition = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
            	"columnName" => "CDPRODUTO",
                "operator" => "=",
                "value" => $row['CDPRODUTO']
            )
        );
        $filterCriteria = new FilterCriteria('pla00600_verificaproduto', $condition);
        $parameterBagColumns = array('CDFILIAL', 'CDPRODUTO');
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $row = $dataSet->getRows();
        if(!empty($row) && $row[0]['IDPRODATIVPR'] !== 'S'){
        	$validate = array(
            	"ERROR" => "Impossível adicionar! Produto não cadastrado para filial ou inativo."
           	);
        } elseif(!empty($row) && $row[0]['IDCNTRESTOQ'] !== 'S'){
        	$validate = array(
        		"ERROR" => "Impossível adicionar! O produto não possui controle de estoque"
        	);
        } 
        return $validate;
    }

	public function retornaMaxNRPRPREVRETI(Request\Row $request, Response $response) {
		try {
			$campos = $request->getParameters();
	        $nrorg = $campos['NRORG'];

			$maxNRPRPREVRETI = $this->get_maxNrPrPrevReti($nrorg, $campos);
			$maxNRPRPREVRETI = array($maxNRPRPREVRETI);
			$response->addDataSet(new DataSet ('maxNRPRPREVRETI', $maxNRPRPREVRETI));

		} catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
	}

    public function get_maxNrPrPrevReti($nrorg, $campos){
        $sql = "SELECT LPAD(NVL(MAX(PR.NRPRPREVRETI), 0)+1, 4, '0') AS NRPRPREVRETI
                  FROM PREVRETI PR
                 WHERE (PR.CDFILIAL   = :P_CDFILIAL)
                   AND (PR.NRORG      = :P_NRORG)
                   AND (PR.CDSERVFILI = :P_CDSERVFILI)
                   AND (PR.DTPREVRETI = :P_DTPREVRETI)
                   AND (PR.IDPREVRETI = :P_IDPREVRETI)
                   AND (PR.NRREQUEXTR = '0000000000')";

        $params = array(
            "P_CDFILIAL"    => $campos['CDFILIAL'],
            "P_NRORG"       => $nrorg,
            "P_CDSERVFILI"  => $campos['CDSERVFILI'],
            "P_DTPREVRETI"  => $campos['DTPREVRETI'],
            "P_IDPREVRETI"  => $campos['IDPREVRETI']
        );

        $result = $this->entityManager->getConnection()->fetchColumn($sql, $params);
        
        return $result ? $result : false;
    }
    
	public function converteParaDateTime($date) {
		$date = explode('/', $date);
		return new DateTime($date[2].'-'.$date[1].'-'.$date[0]);
	}
}
