<?php
namespace RotinaEstoque\Services\Planning;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class HomeService {
  protected $session;
  
  public function __construct($entityManager, $environment, $novoCodigo, $session){
    $this->connection = $entityManager->getConnection();
    $this->environment = $environment;
    $this->novoCodigo = $novoCodigo;
    $this->session = $session;
  }
  
  public function beginTransaction() {
    $this->connection->beginTransaction();
  }
  
  public function commit() {
    $this->connection->commit();
  }
  
  public function rollback() {
    $this->connection->rollback();
  }

  public function planningWithdrawalData($params) {
    $sql = "SELECT PE.IDLANCDTMEN, PE.IDLANCDTMAI, CASE WHEN PG.IDUTILDTPROC = 'S' THEN NVL(PF.DTPROCESSA,SYSDATE) ELSE SYSDATE END AS DTPROCESSA, 
                       NVL(PE.NRDIASSAID,0) AS NRDIASSAID, NVL(PE.NRDIASSAIDPD,0) AS NRDIASSAIDPD, PF.IDLANCDTMENFIL, NVL(PF.NRDIASSAIDFIL,0) AS NRDIASSAIDFIL, PF.IDLANCDTMAIFIL, NVL(PF.NRDIASSAIDPDFIL,0) AS NRDIASSAIDPDFIL,
                       PP.CDFILIAL, PP.IDEXIBESALDOESTRETI, PF.IDESTONLINE, PF.IDCTRLESTQ, PF.IDUTILALMOX, PF.IDUTILLOTE, TP.CDTIPOOPER,
                       PP.IDUTREQUPRAT, PP.CDTPRETIPADR, PP.CDSERVRETPME, NVL(PF.CDTIPCUSTEST, '03') CDTIPCUSTEST, PF.IDPERMVISPOSEST, PP.IDCARDIFRETI
                  FROM PARAMGERAL PG INNER JOIN PARAMFILIAL PF ON PF.NRORG = PG.NRORG AND PG.CDPARGERAL = '0001' 
                                     INNER JOIN PARAMESTOQUE PE ON PE.NRORG = PF.NRORG AND PE.CDPARESTOQ = '1' 
                                     LEFT JOIN TIPOOPERAC TP ON TP.NRORG = PF.NRORG AND TP.IDENTRSAIDOP = 'S' AND TP.CDTIPOOPER = '10' 
                                     LEFT JOIN PARAMPLANEJA PP ON PP.NRORG = PF.NRORG AND PP.CDFILIAL = PF.CDFILIAL 
                  WHERE PF.CDFILIAL = :CDFILIAL AND PF.NRORG = :NRORG";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }

  public function prevreti($params) {
    $sql = "SELECT * FROM PREVRETI WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND DTPREVRETI = :DTPREVRETI AND IDPREVRETI = :IDPREVRETI";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }

  public function servico($params) {
    $sql = "SELECT * FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }

  public function loadMessage($params) {
    $sql = "SELECT * FROM MENSREQU WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND DTMENSREQU = :DTMENSREQU";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }

  public function verifyProduct($params) {
    $sql = "SELECT PF.IDCNTRESTOQ, PF.CDFILIAL, P.NRORG, P.CDPRODUTO, NVL(PF.IDPRODATIV,'A') AS IDPRODATIV, P.IDPRODATIVPR FROM PRODFILI PF, PRODUTO P WHERE P.CDPRODUTO = :CDPRODUTO AND PF.CDFILIAL(+) = :CDFILIAL AND PF.CDPRODUTO (+) = P.CDPRODUTO";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }

  public function calculaRequisicao($params) {
    try{
			$cdoperador  = $params['CDOPERADOR'];
			$cdfilial    = $params['CDFILIAL'];
			$servicos    = $params['SERVICOS'];
			$dtinicial   = $params['DTINICIAL'];
			$dtfinal     = $params['DTFINAL'];
			$dthrcalcpr  = $params['DTHRCALCPR'];
			$peacrescimo = $params['PEACRESCIMO'];
			$stdtinicial = $params['STDTINICIAL'];
			$stdtfinal   = $params['STDTFINAL'];
			$idarredond  = $params['IDARREDOND'];
			$idprodutos  = $params['IDPRODUTOS'];
			$nrorg       = $params['NRORG'];
			$contador    = 'CONSPLAN';
			$nrconsplan  = $this->novoCodigo->getCodeByTableName($contador.$params['CDFILIAL'], $nrorg);
      $this->REQUISICAO_POR_SERVICO_PLA($cdfilial, $servicos, $dtinicial, $dtfinal, $peacrescimo, $cdoperador, $dthrcalcpr,  $stdtinicial, $stdtfinal, $idarredond, $idprodutos, $nrconsplan);
	   	$nrconsplan = array($nrconsplan);
      return $nrconsplan;
    }catch(Exception $e) {
      throw $e; 
    }
  }

	public function REQUISICAO_POR_SERVICO_PLA($cdfilial, $servicos, $dtinicial, $dtfinal, $peacrescimo, $cdoperador, $dthrcalcpr,  $stdtinicial, $stdtfinal, $idarredond, $idprodutos, $nrconsplan) {
    $connection = $this->connection;
    $procedure = new StoredProcedure($connection, 'REQUISICAO_POR_SERVICO_PLA');
    $procedure->addParam(new Param('CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('SERVICOS', Param::PARAM_INPUT, $servicos, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, $peacrescimo, Param::PARAM_TYPE_INT));
    $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('DTHRCALCPR', Param::PARAM_INPUT, $dthrcalcpr, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('NRCONSPLAN', Param::PARAM_INPUT, $nrconsplan, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('STDTINICIAL', Param::PARAM_INPUT, $stdtinicial, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('STDTFINAL', Param::PARAM_INPUT, $stdtfinal, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, $idarredond, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('IDPRODUTOS', Param::PARAM_INPUT, $idprodutos, Param::PARAM_TYPE_STR));
    $procedure->execute();
  }
  
  public function getCdNvProduto() {
    $sql = "SELECT MAX(CDNVPRODUTO)  as CDNVPRODUTO FROM PRODUTO";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute();
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }
  
  public function testConnection() {
    $sql = "SELECT SYSDATE FROM DUAL";
    try{
      $std = $this->connection->prepare($sql);
      $std->execute();
      $result = $std->fetchAll();
      return $result;
    }catch(Exception $e) {
      throw $e; 
    }
  }
  
  public function userInfo() {
        $userInfo = [];
        $sessionInfo = $this->session->get("LOGIN_USER_INFO");
        if(!empty($sessionInfo) && !empty($sessionInfo['USER_DATA'])) {
            $cdfilial = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDFILIAL']);
            $nmfilial = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMFILIAL']);
            $nmacesso = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMACESSO']);
            
            $userInfo = [
                "CDFILIALSELECIONADA" => $cdfilial == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDFILIAL'] : '',
                "NMFILIAL" => $nmfilial == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMFILIAL'] : '',
                "CDPICTPROD" => $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDPICTPROD'],
                "CDPICTPRAT" => $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDPICTPRAT'],
                "NMACESSO" => $nmacesso == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMACESSO'] : ''
            ];
        }
        return new DataSet('userInfo', $userInfo);
    }
}