import { ITekConfig } from '@zeedhi/teknisa-components-common';
import prodEnvironment from './prodEnvironment.json';
import devEnvironment from './devEnvironment.json';

export interface ScmMask extends Object {
  pattern: string;
  nivelMax?: number;
}

const env = process.env.NODE_ENV === 'production' ? prodEnvironment : devEnvironment;
const cdpictprod: string | null = localStorage.getItem('CDPICTPROD');
const cdpictprat: string | null = localStorage.getItem('CDPICTPRAT');
const displayFormat = localStorage.getItem('DATEFORMAT') || 'DD/MM/YYYY';

const decimalPlaces = '5';
const separador = localStorage.getItem('LOGIN_SEPARATOR') || '.,';
const decimalCharacter = separador.substring(2, 1) || ',';
const digitGroupSeparator = separador.substring(0, 1) || '.';

const productMask: ScmMask = {
  pattern: cdpictprod ? cdpictprod.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'N.NN.NN.NNN.NN',
  nivelMax: cdpictprod ? cdpictprod.split('.').length : 5,
};

const plateMask: ScmMask = {
  pattern: cdpictprat ? cdpictprat.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'NN.NN.NN.NNN',
  nivelMax: cdpictprat ? cdpictprat.split('.').length : 5,
};

let envEndPoint = '';
if (env.endPoint !== '') {
  envEndPoint = env.endPoint;
} else if (localStorage.getItem('SERVER_URL')) {
  envEndPoint = process.env.NODE_ENV === 'production' ? `${localStorage.getItem('SERVER_URL')}/backend/index.php`
    : `${localStorage.getItem('SERVER_URL')}/backend/service/index.php`;
} else envEndPoint = '';

const config: ITekConfig = {
  language: 'pt-BR',
  dateFormat: 'YYYY-MM-DD',
  displayFormat,
  displayDateTimeFormat: 'DD/MM/YYYY HH:mm',
  dateTimeFormat: 'YYYY-MM-DDTHH:mm:ss',
  homeUrl: '/',
  monthFormat: 'MM/YY',
  monthDisplayFormat: 'MMM/YY',
  endPoint: envEndPoint,
  metadataEndPoint: env.metadataEndPoint !== '' ? env.metadataEndPoint : `${envEndPoint}/lib_metadata/`,
  env: {
    product: env.product,
    baseUrl: env.baseUrl,
    modules: env.modules,
    loginRoutePath: env.modules.login.routePath,
    loginBackgroundImgUrl: env.modules.login.backgroundImgUrl,
    loginEndPoint: env.modules.login.endPoint !== '' ? env.modules.login.endPoint
      : `${localStorage.getItem('SERVER_URL')}/backend_login/index.php` || '',
  },
  saveGridLayoutsEndPoint: env.saveGridLayoutsEndPoint,
  loadGridLayoutsEndPoint: env.loadGridLayoutsEndPoint,
  masks: {
    numberMask: {
      decimalCharacter: `${decimalCharacter}`,
      digitGroupSeparator: `${digitGroupSeparator}`,
      currencySymbol: '',
      decimalPlaces: `${decimalPlaces}`,
    },
    productMask,
    plateMask,
  },
};

export default config;
