import {
Loader,
Metadata,
I18n,
} from '@zeedhi/core';
import {
IterableComponentRender,
AutoNumeric,
Text,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class SummaryController {
  private PlanningWithdrawalController = Loader.getInstance('PlanningWithdrawalController');

  public serviceSelected: string = '';

  public CDFILIAL: string = '';

  public NRRETIRADA: any;

  public DTPOSESTCON: string = '';

  public summaryPageVisibility: boolean = false;

  public evolved: string = 'EVOLUÍDO';

  public notEvolved: string = 'NÃO EVOLUÍDO';

  public returnToMainPage() {
    const summaryMainComponent = Metadata.getInstance<IterableComponentRender>('summaryIterable');
    const summaryEmptyText = Metadata.getInstance<Text>('textForSummaryCondition');
    this.summaryPageVisibility = false;
    this.PlanningWithdrawalController.mainPageVisibility = !this.summaryPageVisibility;
    this.PlanningWithdrawalController.addWaybillButtonVisibility = true;
    summaryMainComponent.isVisible = true;
    summaryEmptyText.isVisible = false;
  }

  public async openSummaryPage({ component }: any) {
    this.PlanningWithdrawalController.addWaybillButtonVisibility = false;
    this.PlanningWithdrawalController.mainPageVisibility = false;
    this.summaryPageVisibility = true;
    const serviceData = component.name.split('_');
    const CDFILIAL = serviceData[0];
    const NRRETIRADA = serviceData[1];
    this.CDFILIAL = CDFILIAL;
    this.NRRETIRADA = NRRETIRADA;
    this.DTPOSESTCON = serviceData[2].substring(0, 10);
    const serviceName = serviceData[3];
    const serviceCode = serviceData[4];
    this.serviceSelected = `${serviceCode} - ${serviceName}`;

    this.loadSummaryData();
  }

  public loadSummaryData() {
    const summaryMainComponent = Metadata.getInstance<IterableComponentRender>('summaryIterable');
    const summaryEmptyText = Metadata.getInstance<Text>('textForSummaryCondition');
    const conditions = {
      CDFILIAL: this.CDFILIAL,
      NRRETIRADA: this.NRRETIRADA,
      DTPOSESTCON: this.DTPOSESTCON,
    };
    summaryMainComponent.datasource.filter = conditions;
    summaryMainComponent.datasource.get().then((response: any) => {
      if (response.length === 0) {
        summaryEmptyText.isVisible = true;
        summaryMainComponent.isVisible = false;
      }
    });
  }

  public stylizeSummaryText({ component }: any) {
    const evolved = I18n.translate(this.evolved);
    const notEvolved = I18n.translate(this.notEvolved);
    const status = component.name.split('_')[1];
    if (status === 'S') {
      component.text = `<span style = 'background-color: #dee2e6; color: green; font-size: 13px;'>${evolved}</span>`;
    }
    if (status === 'N') {
      component.text = `<span style = 'background-color: #dee2e6; color: red; font-size: 13px;'>${notEvolved}</span>`;
    }
  }

  // backup sobre como colocar uma mascara personalizada caso necessário
  // public setMaskForCDARVPROD({ component }: any) {
  //   console.warn(component.name);
  //   const text = component.text as string;
  //   component.text = Mask.getValueWithMask(text, '#.#.#.##.###.##');
  // }

  public setMaskForFloat({ component }: any) {
    try {
      const number = component.name.split('_');
      const maskedValue = AutoNumeric.format(number[0], { decimalPlaces: 5 });
      component.text = `${component.text} ${maskedValue}`;
    } catch (error) {
      component.text = `${component.text} 0.0`;
    }
  }
}
