import {
Metadata,
Loader,
Http,
I18n,
dayjs,
IEventParam,
} from '@zeedhi/core';
import {
Modal,
ModalService,
Select,
Form,
TextInput,
IterableComponentRender,
Time,
DateRange,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class AddserviceProductsModal {
  private Utilities = Loader.getInstance('Utilities');

  private ServiceController = Loader.getInstance('ServiceController');

  private WaybillLaunchController = Loader.getInstance('WaybillLaunchController');

  private basicModal!: Modal;

  public addTypePlate: boolean = false;

  public async createBasicUsageModal() {
    this.basicModal = await ModalService.createFromJson('planning/modals/addServiceModal', true);
  }

  public destroyBasicUsageModal() {
    this.basicModal?.destroy();
  }

  public openBasicModal() {
    this.basicModal.show();
  }

  public closeBasicModal() {
    this.clearModalFields();
    this.basicModal.hide();
  }

  public newProductAdded: string = 'Novo produto adicionado com sucesso.';

  public saveProduct() {
    const form = Metadata.getInstance<Form>('addNewProductForm');
    const disheType = Metadata.getInstance<Select>('disheTypeNewProduct');
    const dishes = Metadata.getInstance<Select>('selectDisheNewProduct');
    const product = this.ServiceController.productSelectedData;
    const plates = this.ServiceController.arrayPlates;
    const formServiceData = this.ServiceController.formSelectedData;
    const arrayProduct = [this.ServiceController.arrayProduct];
    let paramsQuery = this.ServiceController.serviceSelectedData;
    switch (paramsQuery.IDUTRETIPRAT) {
      case 'N':
        disheType.clearValidations();
        dishes.clearValidations();
        break;
      case 'S':
        if (!product.CDPRATO) {
          disheType.addValidation('required');
          dishes.addValidation('required');
        } else {
          disheType.clearValidations();
          dishes.clearValidations();
        }
        break;
      default:
        break;
    }
    const validation = form.validate();
    if (validation) {
      this.fillParamsQuery(paramsQuery, formServiceData);
      let booExistPlate = false;
      if (paramsQuery.IDUTRETIPRAT === 'S') {
        if (!product.CDPRATO) {
          const dishe = Metadata.getInstance<Select>('selectDisheNewProduct').value.split('+');
          const [CDPRATO, CDFILAUXPRAT] = dishe;
          paramsQuery.CDTIPOPRAT = Metadata.getInstance<Select>('disheTypeNewProduct').value;
          paramsQuery.CDPRATO = CDPRATO;
          paramsQuery.CDFILAUXPRAT = CDFILAUXPRAT;
          booExistPlate = this.containsCDPRATOAndCDTIPOPRAT(plates, CDPRATO, paramsQuery.CDTIPOPRAT);
        } else {
          paramsQuery = { ...product, ...paramsQuery };
        }
      } else {
        paramsQuery.CDTIPOPRAT = '';
        paramsQuery.CDPRATO = '';
        paramsQuery.CDFILAUXPRAT = '';
      }
      if (!this.containsCDPRODUTOValue(arrayProduct[0], paramsQuery.CDPRODUTO)) {
        Http.post('/verifyProduct', { CDFILIAL: paramsQuery.CDFILIAL, CDPRODUTO: paramsQuery.CDPRODUTO }).then((callbackVerify) => {
          if (callbackVerify.data.dataset.params.length && callbackVerify.data.dataset.params[0].IDPRODATIVPR === 'S') {
            if (callbackVerify.data.dataset.params[0].IDCNTRESTOQ === 'S') {
              Http.post('/retornaMaxNRPRPREVRETI', paramsQuery).then((callbackNR) => {
                const [maxNRPRPREVRETI] = callbackNR.data.dataset.maxNRPRPREVRETI;
                paramsQuery.NRPRPREVRETI = maxNRPRPREVRETI;
                if (!booExistPlate) {
                  Http.post('/createProduct', {
                    requestType: 'Row',
                    row: paramsQuery,
                  }).then((response) => {
                    if (response.data.error) {
                      this.Utilities.failMessage(I18n.translate(response.data.error));
                    } else {
                      const newProductAdded = I18n.translate(this.newProductAdded);
                      this.Utilities.successMessage(newProductAdded);
                      this.closeBasicModal();
                      this.reloadDataSources();
                    }
                  });
                } else {
                  this.Utilities.failMessage(I18n.translate('prato ja cadastrado.'));
                }
              });
            } else {
              this.Utilities.failMessage(I18n.translate('Impossível adicionar! O produto não possui controle de estoque.'));
            }
          } else {
            this.Utilities.failMessage(I18n.translate('Produto não cadastrado para unidade ou inativo.'));
          }
        });
      } else {
        this.Utilities.failMessage(I18n.translate('Produto Ja cadastrado.'));
      }
    }
  }

  public async productsDataSource({ component }: any) {
    Http.post('/validGrupOper').then(async (callbackValidGrupOper) => {
      if (!callbackValidGrupOper.data.dataset.denied && this.ServiceController.userAuthCalc) {
        await this.openBasicModal();
        const disheType = Metadata.getInstance<Select>('disheTypeNewProduct');
        const dishes = Metadata.getInstance<Select>('selectDisheNewProduct');
        switch (component.name) {
          case 'addProductButton':
            this.addTypePlate = false;
            disheType.isVisible = false;
            dishes.isVisible = false;
            break;
          case 'addProductInDisheButton':
            this.addTypePlate = true;
            disheType.isVisible = true;
            dishes.isVisible = true;
            break;
          default:
            break;
        }
        const product = Metadata.getInstance<Select>('selectProductNewProduct');
        const branch = Metadata.getInstance<Select>('unitsForWithdrawal');
        const conditions = {
          NRORG: localStorage.getItem('NRORG'),
          CDNVPRODUTO: localStorage.getItem('CDNVPRODUTO'),
          CDFILIAL: branch.value.CDFILIAL,
        };
        product.datasource.filter = conditions;
        product.datasource.get();
        Metadata.getInstance<DateRange>('addDTINCLUSAO').value = this.ServiceController.DTPREVRETI;
        this.convertToTimeString(this.ServiceController.HRINICATEN);
      } else {
        this.Utilities.failMessage(I18n.translate('Operador Não Autorizado.'));
      }
    });
  }

  public populateSgUnityField({ component }: any) {
    const unity = Metadata.getInstance<TextInput>('productUnityAddNewProduct');
    component.datasource.data.forEach((item: any) => {
      if (item.CDPRODUTO === component.value) {
        unity.value = item.SGUNIDADE;
      }
    });
  }

  public reloadDataSources() {
    const products = Metadata.getInstance<IterableComponentRender>('iterableProducts');
    const withdrawalsCards = Metadata.getInstance<IterableComponentRender>('iterableWaybillCard');
    const arrayProduct = [this.ServiceController.arrayProduct];
    products.reload();
    withdrawalsCards.reload();
    if (!arrayProduct[0].some((item: { CDPRODUTO: string }) => item.CDPRODUTO !== undefined)) {
      this.ServiceController.loadServiceBillData();
    }
  }

  public containsCDPRODUTOValue(list: { [key: string]: any }[], value: any): boolean {
  if (!Array.isArray(list)) {
    return false;
  }
    return list.some((obj) => obj.CDPRODUTO === value);
  }

  public containsCDPRATOAndCDTIPOPRAT(list: { CDPRATO: string, CDTIPOPRAT: string }[], cdPrato: string, cdTipoPrato: string): boolean {
    if (!Array.isArray(list)) {
      return false;
    }
    return list.some((obj) => obj.CDPRATO === cdPrato && obj.CDTIPOPRAT === cdTipoPrato);
  }

  public setWithdrawalDate({ component }: any) {
    const withdrawalDate = Metadata.getInstance<DateRange>('addDTINCLUSAO');
    withdrawalDate.value = component.value;
  }

  public setHRLIMITERETI({ component }: IEventParam<Time>) {
    const instance = Metadata.getInstance<Time>('HRLIMITERETI');
    const newValue = dayjs(component.value, 'hhmm').format(instance.displayFormat);
    instance.value = newValue;
  }

  private clearModalFields() {
    const disheType = Metadata.getInstance<Select>('disheTypeNewProduct');
    disheType.clearValidations();
    disheType.value = null;
    const dishes = Metadata.getInstance<Select>('selectDisheNewProduct');
    dishes.clearValidations();
    dishes.value = null;
    Metadata.getInstance<Select>('selectProductNewProduct').value = null;
    Metadata.getInstance<Select>('qtPredictedNewProduct').value = 0;
    Metadata.getInstance<Select>('qtExtraNewProduct').value = 0;
    Metadata.getInstance<Select>('qtReturnedNewProduct').value = 0;
    Metadata.getInstance<Select>('qtConsumedNewProduct').value = 0;
    Metadata.getInstance<TextInput>('productUnityAddNewProduct').value = '';
  }

  private fillParamsQuery(paramsQuery: any, formServiceData: any) {
    paramsQuery.CDPRODUTO = Metadata.getInstance<Select>('selectProductNewProduct').value;
    paramsQuery.QTPREVRETI = Metadata.getInstance<Select>('qtPredictedNewProduct').value;
    paramsQuery.QTPREVEXTRA = Metadata.getInstance<Select>('qtExtraNewProduct').value;
    paramsQuery.QTPREVDEVOL = Metadata.getInstance<Select>('qtReturnedNewProduct').value;
    paramsQuery.QTPREVLIB = Metadata.getInstance<Select>('qtConsumedNewProduct').value;
    paramsQuery.CDOPERADOR = localStorage.getItem('CDOPERADOR');
    paramsQuery.NRREQUEXTR = '0000000000';
    paramsQuery.IDATIVO = 'S';
    paramsQuery.NRORGINCLUSAO = localStorage.getItem('NRORG');
    paramsQuery.DTPREVRETI = formServiceData.DTPREVRETI;
    paramsQuery.DTINCLUSAO = Metadata.getInstance<DateRange>('addDTINCLUSAO').value;
    paramsQuery.DTLIMITERETI = Metadata.getInstance<DateRange>('addDTINCLUSAO').value;
    paramsQuery.HRLIMITERETI = Metadata.getInstance<Time>('HRLIMITERETI').value.replace(':', '');
    paramsQuery.CDOPERINCLUSAO = localStorage.getItem('CDOPERADOR');
    paramsQuery.IDPREVRETI = formServiceData.IDPREVRETI;
  }

  public extraCalculation({ component }: any) {
    const extra = component;
    const consumed = Metadata.getInstance<TextInput>('qtConsumedNewProduct');
    const predicted = Metadata.getInstance<TextInput>('qtPredictedNewProduct');
    const returned = Metadata.getInstance<TextInput>('qtReturnedNewProduct');

    if (extra.value === undefined) {
      extra.value = 0;
    }

    if (typeof extra.value === 'number') {
      returned.value = 0;
      consumed.value = predicted.value + extra.value;
    } else {
      extra.value = 0;
      returned.value = 0;
      consumed.value = predicted.value;
    }
  }

  public returnedCalculation({ component }: any) {
    const returned = component;
    const predicted = Metadata.getInstance<TextInput>('qtPredictedNewProduct').value;
    const extra = Metadata.getInstance<TextInput>('qtExtraNewProduct');
    const consumed = Metadata.getInstance<TextInput>('qtConsumedNewProduct');
    if (returned.value === undefined) {
      returned.value = 0;
    }
    const condition = consumed.value < predicted;

    if (returned.value > predicted && typeof returned.value === 'number') {
      returned.value = predicted;
      extra.value = 0;
      consumed.value = 0;
    } else if (condition && typeof returned.value === 'number') {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    } else {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    }
  }

  public consumedCalculation({ component }: any) {
    const consumed = component;
    const predicted = Metadata.getInstance<TextInput>('qtPredictedNewProduct').value;
    const extra = Metadata.getInstance<TextInput>('qtExtraNewProduct');
    const returned = Metadata.getInstance<TextInput>('qtReturnedNewProduct');
    let condition = false;
    if (consumed.value === undefined) {
      consumed.value = 0;
    }
    condition = consumed.value < predicted ? condition = false : condition = true;

    if (condition === false) {
      extra.value = 0;
      returned.value = predicted - consumed.value;
    }
    if (typeof consumed.value === 'number' && condition === true) {
      returned.value = 0;
      extra.value = consumed.value - predicted - returned.value;
    }
  }

  public convertToTimeString(hhmm: string) {
    const hours = hhmm.substring(0, 2);
    const minutes = hhmm.substring(2, 4);
    const limitFormat = `${hours}:${minutes}`;
    const instance = Metadata.getInstance<Time>('HRLIMITERETI');
    instance.value = limitFormat;
  }
}
