import {
Loader,
Metadata,
IEventParam,
Http,
I18n,
} from '@zeedhi/core';
import {
Modal,
ModalService,
TextInput,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class MessageModalController {
  private basicModal!: Modal;

  private ServiceController = Loader.getInstance('ServiceController');

  private Utilities = Loader.getInstance('Utilities');

  public textMessage: any;

  public authOperator: boolean = false;

  public async createBasicUsageModal() {
    this.basicModal = await ModalService.createFromJson('planning/modals/MessageProductsModal', true);
  }

  public destroyBasicUsageModal() {
    this.basicModal?.destroy();
  }

  public openBasicModal() {
    this.basicModal.show();
  }

  public closeBasicModal() {
    this.basicModal.hide();
  }

  public async openMessage() {
    await this.openBasicModal();
    await this.verifyAuth();
    this.textMessage = this.ServiceController.messageData?.DSOBSREQU;
    const textArea = Metadata.getInstance<TextInput>('messageData');
    textArea.value = this.textMessage ? this.textMessage : '';
  }

  public updateMessage() {
    const textMessageParam = Metadata.getInstance<TextInput>('messageData').value;
    const { messageData } = this.ServiceController;
    const params = this.setMessageParams(messageData, textMessageParam);
    if (this.authOperator) {
      Http.post('/createMessage', {
        requestType: 'Row',
        row: params,
      }).then((response) => {
        if (response.data.error) {
          this.Utilities.failMessage(I18n.translate(response.data.error));
        } else {
          this.Utilities.successMessage(I18n.translate('Mensagem atualizada'));
          this.ServiceController.messageData.DSOBSREQU = textMessageParam;
          this.closeBasicModal();
        }
      });
    } else {
      this.closeBasicModal();
      this.Utilities.failMessage(I18n.translate('Operador Não Autorizado'));
    }
  }

  public setMessageParams(messageData: any, textMessageParam: any): any {
    const params = {
      NRORG: messageData.NRORG,
      CDFILIAL: messageData.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      CDSERVFILI: messageData.CDSERVFILI,
      DTMENSREQU: messageData.DTMENSREQU ? messageData.DTMENSREQU.split(' ')[0] : messageData.DTPREVRETI,
      DSOBSREQU: textMessageParam,
      NRSEQUREQU: '0000000000',
      DTINCLUSAO: messageData.DTINCLUSAO ? messageData.DTINCLUSAO.split(' ')[0] : null,
      DTULTATU: messageData.DTULTATU ? messageData.DTULTATU.split(' ')[0] : null,
      IDATIVO: 'S',
      NRORGINCLUSAO: messageData.NRORGINCLUSAO ? messageData.NRORGINCLUSAO : messageData.NRORG,
      NRORGULTATU: messageData.NRORGULTATU ? messageData.NRORGULTATU : messageData.NRORG,
    };
    return params;
  }

  public setMessageValue({ component }: IEventParam<TextInput>) {
    const instance = Metadata.getInstance<TextInput>('messageData');
    instance.value = component.value;
  }

  public verifyAuth() {
    Http.post('/validGrupOper').then((callbackValidGrupOper) => {
      if (!callbackValidGrupOper.data.dataset.denied) {
        this.authOperator = true;
      }
    });
  }
}
