import {
Metadata,
Loader,
Http,
I18n,
} from '@zeedhi/core';
import {
Modal,
ModalService,
TextInput,
Select,
Number as NumberInput,
DialogService,
IterableComponentRender,
} from '@zeedhi/common';
import {
} from '@zeedhi/teknisa-components-common';

export class ProductsUpdateModalController {
  private Utilities = Loader.getInstance('Utilities');

  private PlanningWithdrawalController = Loader.getInstance('PlanningWithdrawalController');

  public productsIterable: any;

  public instanceReference: any;

  private productUpdateModal!: Modal;

  public async createBasicUsageModal() {
    this.productUpdateModal = await ModalService.createFromJson('planning/modals/productsUpdateModal', true);
  }

  public destroyBasicUsageModal() {
    this.productUpdateModal?.destroy();
  }

  public openBasicModal() {
    this.productUpdateModal.show();
  }

  public closeBasicModal() {
    this.productUpdateModal.hide();
  }

  public productDataSourceToBeUpdated: any;

  public collapseCardExpanded: any;

  public productUpdated: string = 'Produto atualizado com sucesso.';

  public updateProductTitle: string = 'Atualizar produto?';

  public updateProductText: string = 'Deseja atualizar o produto selecionado?';

  public async clickProductsCard({ component }: any) {
    await this.openBasicModal();
    this.productsIterable = Metadata.getInstance<IterableComponentRender>('iterableProducts');
    const conditions = component.name.split('_');
    this.productsIterable.datasource.data.forEach((item: any, index: any) => {
      if (item.CDFILIAL === conditions[0] && item.NRCARDAPIO === conditions[1] && item.NRRETIRADA === conditions[2] && item.NRSEQUITRETI === conditions[3]) {
        this.productDataSourceToBeUpdated = this.productsIterable.datasource.data[index];
        this.productDataSourceToBeUpdated.VALIDAR_ESTOQUE = this.PlanningWithdrawalController.unitParams.VALIDAR_ESTOQUE;
        this.productDataSourceToBeUpdated.DTRETIRADA = this.PlanningWithdrawalController.unitParams.DTRETIRADA;
        this.instanceReference = 'products';
        Metadata.getInstance<TextInput>('idProduct').value = item.CDPRODUTOFORMAT;
        Metadata.getInstance<TextInput>('productName').value = item.NMPRODUTO;
        Metadata.getInstance<TextInput>('productUnity').value = item.SGUNIDADE;
        Metadata.getInstance<TextInput>('productPredicted').value = item.QTPREVISRETI;
        Metadata.getInstance<TextInput>('productExtra').value = item.QTEXTRARETI;
        Metadata.getInstance<TextInput>('productReturned').value = item.QTDEVOLVRETI;
        Metadata.getInstance<TextInput>('productConsumed').value = item.QTPREVLIB;
        Metadata.getInstance<TextInput>('unitaryValue').value = item.VRUNILANCTO;
        Metadata.getInstance<Select>('consumingType').value = item.NMTIPORETI;
        Metadata.getInstance<TextInput>('stockAmount').value = item.QTESTOQDIA;
      }
    });
  }

  public async clickProductsInDishesCard({ component }: any) {
    await this.openBasicModal();
    const iterable = component.name.split('-');
    const conditionsUpd = component.name.split('_');
    const product = Metadata.getInstance<IterableComponentRender>(`iterableProductsInDishesTypes_${iterable[2]}${iterable[3]}${iterable[4]}`);
    this.collapseCardExpanded = Metadata.getInstance<IterableComponentRender>(`collapseCard-${iterable[2]}-${iterable[3]}-${iterable[4]}-${iterable[5]}`);
    product.datasource.data.forEach((item: any, index: any) => {
      if (item.CDFILIAL === conditionsUpd[0] && item.NRCARDAPIO === conditionsUpd[1] && item.NRRETIRADA === conditionsUpd[2] && item.NRSEQUITRETI === conditionsUpd[3]) {
        this.productDataSourceToBeUpdated = product.datasource.data[index];
        this.productDataSourceToBeUpdated.VALIDAR_ESTOQUE = this.PlanningWithdrawalController.unitParams.VALIDAR_ESTOQUE;
        this.productDataSourceToBeUpdated.DTRETIRADA = this.PlanningWithdrawalController.unitParams.DTRETIRADA;
        this.instanceReference = 'dishes';
        Metadata.getInstance<TextInput>('idProduct').value = item.CDPRODUTOFORMAT;
        Metadata.getInstance<TextInput>('productName').value = item.NMPRODUTO;
        Metadata.getInstance<TextInput>('productUnity').value = item.SGUNIDADE;
        Metadata.getInstance<TextInput>('productPredicted').value = item.QTPREVISRETI;
        Metadata.getInstance<TextInput>('productExtra').value = item.QTEXTRARETI;
        Metadata.getInstance<TextInput>('productReturned').value = item.QTDEVOLVRETI;
        Metadata.getInstance<TextInput>('productConsumed').value = item.QTRETIRADA;
        Metadata.getInstance<TextInput>('unitaryValue').value = item.VRUNILANCTO;
        Metadata.getInstance<Select>('consumingType').value = item.NMTIPORETI;
        Metadata.getInstance<TextInput>('stockAmount').value = item.QTESTOQDIA;
      }
    });
  }

  public updateProduct() {
    this.productDataSourceToBeUpdated.QTEXTRARETI = Metadata.getInstance<NumberInput>('productExtra').value;
    this.productDataSourceToBeUpdated.QTDEVOLVRETI = Metadata.getInstance<NumberInput>('productReturned').value;
    this.productDataSourceToBeUpdated.QTRETIRADA = Metadata.getInstance<NumberInput>('productConsumed').value;
    this.productDataSourceToBeUpdated.NMTIPORETI = Metadata.getInstance<Select>('consumingType').value;
    this.productDataSourceToBeUpdated.CDFILIMOVI = Metadata.getInstance<Select>('unitsForWithdrawal').value.CDFILIAL;
    this.productDataSourceToBeUpdated.NRLANCESTQ = this.PlanningWithdrawalController.nrlancestq;
    // const productsInDishesIterable = Metadata.getInstance<IterableComponentRender>('iterableCollapseCardDishesTypes');
    Http.post('/updateProduct', {
      requestType: 'Row',
      row: this.productDataSourceToBeUpdated,
    }).then((response) => {
      if (response.data.error) {
        if (this.instanceReference === 'dishes') {
          // productsInDishesIterable.reload();
        } else {
          this.productsIterable.reload();
        }
        this.Utilities.warnMessage(response.data.error);
        DialogService.hide();
      } else {
        if (this.instanceReference === 'dishes') {
          // productsInDishesIterable.reload();
        } else {
          this.productsIterable.reload();
        }
        const productUpdated = I18n.translate(this.productUpdated);
        this.Utilities.successMessage(productUpdated);
        DialogService.hide();
        this.closeBasicModal();
      }
    });
    /* copiar os valores iniciais da modal de update, if exception, retornar os valores para os inputs
    reset filter no cancelar
    */
  }

  public closeConfirmationDialog: any = {
    click: () => DialogService.hide(),
  };

  public updateProductDialog() {
    const dialogParams = {
      updateProductTitle: I18n.translate(this.updateProductTitle),
      updateProductText: I18n.translate(this.updateProductText),
    };
    this.Utilities.showConfirmationDialog(dialogParams.updateProductTitle, dialogParams.updateProductText, this.closeConfirmationDialog, this.updateProductButton);
  }

  public updateProductButton: any = {
    click: () => this.updateProduct(),
  };

  public extraCalculation({ component }: any) {
    const extra = component;
    const consumed = Metadata.getInstance<NumberInput>('productConsumed');
    const predicted = Metadata.getInstance<NumberInput>('productPredicted');
    const returned = Metadata.getInstance<NumberInput>('productReturned');

    if (extra.value === undefined) {
      extra.value = 0;
    }

    if (typeof extra.value === 'number') {
      returned.value = 0;
      consumed.value = predicted.value;
    } else {
      extra.value = 0;
      returned.value = 0;
      consumed.value = predicted.value;
    }
  }

  public returnedCalculation({ component }: any) {
    const returned = component;
    const predicted = Metadata.getInstance<NumberInput>('productPredicted').value;
    const extra = Metadata.getInstance<NumberInput>('productExtra');
    const consumed = Metadata.getInstance<NumberInput>('productConsumed');
    if (returned.value === undefined) {
      returned.value = 0;
    }
    const condition = consumed.value < predicted;

    if (returned.value > predicted && typeof returned.value === 'number') {
      returned.value = predicted;
      extra.value = 0;
      consumed.value = 0;
    } else if (condition && typeof returned.value === 'number') {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    } else {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    }
  }

  public consumedCalculation({ component }: any) {
    const consumed = component;
    const predicted = Metadata.getInstance<NumberInput>('productPredicted').value;
    const extra = Metadata.getInstance<NumberInput>('productExtra');
    const returned = Metadata.getInstance<NumberInput>('productReturned');
    let condition = false;
    if (consumed.value === undefined) {
      consumed.value = 0;
    }
    condition = consumed.value < predicted ? condition = false : condition = true;

    if (condition === false) {
      extra.value = 0;
      returned.value = predicted - consumed.value;
    }
    if (typeof consumed.value === 'number' && condition === true) {
      returned.value = 0;
      extra.value = consumed.value - predicted - returned.value;
    }
  }
}
