import {
Metadata,
Loader,
Http,
I18n,
dayjs,
IEventParam,
} from '@zeedhi/core';
import {
Modal,
ModalService,
TextInput,
// Select,
Number as NumberInput,
DialogService,
DateRange,
Time,
IterableComponentRender,
} from '@zeedhi/common';
import {
} from '@zeedhi/teknisa-components-common';

export class UpdateServiceModalController {
  private Utilities = Loader.getInstance('Utilities');

  private PlanningWithdrawalController = Loader.getInstance('PlanningWithdrawalController');

  private ServiceController = Loader.getInstance('ServiceController');

  public productsIterable: any;

  public productParams: any;

  public authOperator: boolean = false;

  public instanceReference: any;

  private productUpdateModal!: Modal;

  public async createBasicUsageModal() {
    this.productUpdateModal = await ModalService.createFromJson('planning/modals/ServiceUpdateModal', true);
  }

  public destroyBasicUsageModal() {
    this.productUpdateModal?.destroy();
  }

  public openBasicModal() {
    this.productUpdateModal.show();
  }

  public closeBasicModal() {
    this.productUpdateModal.hide();
  }

  public productDataSourceToBeUpdated: any;

  public collapseCardExpanded: any;

  public productUpdated: string = 'Produto atualizado com sucesso.';

  public updateProductTitle: string = 'Atualizar produto?';

  public updateProductText: string = 'Deseja atualizar o produto selecionado?';

  public async clickProductsCard({ component }: any) {
    if (!this.ServiceController.userAuthCalc) {
      this.Utilities.failMessage(I18n.translate('Operador logado difere do responsável pela requisição.'));
    } else {
      await this.openBasicModal();
      this.productsIterable = Metadata.getInstance<IterableComponentRender>('iterableProducts');
      const conditions = component.name.split('_');
      const CDPRODUTO = conditions[2];
      const arrayProduct = [this.ServiceController.arrayProduct];
      this.productParams = arrayProduct[0].find((item: any) => item.CDPRODUTO === CDPRODUTO);
      const Mask = (localStorage.getItem('CDPICTPROD') && localStorage.getItem('CDPICTPROD') !== '')
        ? localStorage.getItem('CDPICTPROD')?.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'N.NN.NN.NNN.NN';
      const prodMask = this.Utilities.applyMask(Mask, this.productParams.CDPRODUTO);
      Metadata.getInstance<TextInput>('idProduct').value = prodMask;
      Metadata.getInstance<TextInput>('productName').value = this.productParams.NMPRODUTO;
      Metadata.getInstance<TextInput>('productUnity').value = this.productParams.SGUNIDADE;
      Metadata.getInstance<TextInput>('productPredicted').value = this.productParams.QTPREVRETI ? this.productParams.QTPREVRETI : '0.00000';
      Metadata.getInstance<TextInput>('productExtra').value = this.productParams.QTPREVEXTRA ? this.productParams.QTPREVEXTRA : '0.00000';
      Metadata.getInstance<TextInput>('productReturned').value = this.productParams.QTPREVDEVOL ? this.productParams.QTPREVDEVOL : '0.00000';
      Metadata.getInstance<TextInput>('productConsumed').value = this.productParams.QTPREVLIB ? this.productParams.QTPREVLIB : '0.00000';
      Metadata.getInstance<DateRange>('DTINCLUSAO').value = this.productParams.DTLIMITERETI;
      this.convertToTimeString(this.productParams.HRLIMITERETI);
    }
  }

  public updateProduct() {
    this.productParams.QTPREVRETI = Metadata.getInstance<TextInput>('productPredicted').value;
    this.productParams.QTPREVEXTRA = Metadata.getInstance<TextInput>('productExtra').value;
    this.productParams.QTPREVDEVOL = Metadata.getInstance<TextInput>('productReturned').value;
    this.productParams.QTPREVLIB = Metadata.getInstance<TextInput>('productConsumed').value;
    this.productParams.DTLIMITERETI = Metadata.getInstance<DateRange>('DTINCLUSAO').value;
    this.productParams.HRLIMITERETI = Metadata.getInstance<Time>('productHRLIMITERETI').value.replace(':', '');
    this.productParams.DTULTATU = null;
    this.productParams.DTPREVRETI = this.productParams.DTPREVRETI?.split(' ')[0];
    this.productParams.DTINCLUSAO = this.productParams.DTINCLUSAO?.split(' ')[0];
    Http.post('/createProduct', {
      requestType: 'Row',
      row: this.productParams,
    }).then((response) => {
      if (response.data.error) {
        this.Utilities.failMessage(I18n.translate(response.data.error));
      } else {
        this.Utilities.successMessage(I18n.translate(this.productUpdated));
      }
    });
    this.closeBasicModal();
    DialogService.hide();
  }

  public closeConfirmationDialog: any = {
    click: () => DialogService.hide(),
  };

  public updateProductDialog() {
    const dialogParams = {
      updateProductTitle: I18n.translate(this.updateProductTitle),
      updateProductText: I18n.translate(this.updateProductText),
    };
    this.Utilities.showConfirmationDialog(dialogParams.updateProductTitle, dialogParams.updateProductText, this.closeConfirmationDialog, this.updateProductButton);
  }

  public updateProductButton: any = {
    click: () => this.updateProduct(),
  };

  public extraCalculation({ component }: any) {
    const extra = component;
    const consumed = Metadata.getInstance<NumberInput>('productConsumed');
    const predicted = Metadata.getInstance<NumberInput>('productPredicted');
    const returned = Metadata.getInstance<NumberInput>('productReturned');

    if (extra.value === undefined) {
      extra.value = 0;
    }

    if (typeof extra.value === 'number') {
      returned.value = 0;
      consumed.value = predicted.value + extra.value;
    } else {
      extra.value = 0;
      returned.value = 0;
      consumed.value = predicted.value;
    }
  }

  public returnedCalculation({ component }: any) {
    const returned = component;
    const predicted = Metadata.getInstance<NumberInput>('productPredicted').value;
    const extra = Metadata.getInstance<NumberInput>('productExtra');
    const consumed = Metadata.getInstance<NumberInput>('productConsumed');
    if (returned.value === undefined) {
      returned.value = 0;
    }
    const condition = consumed.value < predicted;

    if (returned.value > predicted && typeof returned.value === 'number') {
      returned.value = predicted;
      extra.value = 0;
      consumed.value = 0;
    } else if (condition && typeof returned.value === 'number') {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    } else {
      extra.value = 0;
      consumed.value = predicted - returned.value;
    }
  }

  public consumedCalculation({ component }: any) {
    const consumed = component;
    const predicted = Metadata.getInstance<NumberInput>('productPredicted').value;
    const extra = Metadata.getInstance<NumberInput>('productExtra');
    const returned = Metadata.getInstance<NumberInput>('productReturned');
    let condition = false;
    if (consumed.value === undefined) {
      consumed.value = 0;
    }
    condition = consumed.value < predicted ? condition = false : condition = true;

    if (condition === false) {
      extra.value = 0;
      returned.value = predicted - consumed.value;
    }
    if (typeof consumed.value === 'number' && condition === true) {
      returned.value = 0;
      extra.value = consumed.value - predicted - returned.value;
    }
  }

  public convertToTimeString(hhmm: string) {
    const hours = hhmm.substring(0, 2);
    const minutes = hhmm.substring(2, 4);
    const limitFormat = `${hours}:${minutes}`;
    const instance = Metadata.getInstance<Time>('productHRLIMITERETI');
    instance.value = limitFormat;
  }

  public setHRLIMITERETI({ component }: IEventParam<Time>) {
    const instance = Metadata.getInstance<Time>('productHRLIMITERETI');
    const newValue = dayjs(component.value, 'hhmm').format(instance.displayFormat);
    instance.value = newValue;
  }
}
